<?php

require_once('config.php');
require_once('curl_util.php');

class OPENDS {
    private $opends_token = NULL;

    private $curl_tool = NULL;

    public function __construct($token, $domain) {
        $this->opends_token = $token;
        $this->curl_tool = new CurlUtil();
        if (NEED_REDIRECT){
            $real_url = $this->service_uri($domain);
            echo $real_url;
            $this->curl_tool->set_api_url($real_url);
        }
    }

    public function create_ds($name) {
        $res = $this->curl_tool->Post(
            DS_CREATE,
            array(
                'access_token' => $this->opends_token,
                'name' => $name,
                'type' => 'opends'
            ),
            array());
        return $res['ds_id'];
    }

    public function list_ds() {
        $res = $this->curl_tool->Get(
            DS_LIST,
            array('access_token' => $this->opends_token));
        return $res['data_source'];
    }

    public function delete_ds($ds_id) {
        $res = $this->curl_tool->Get(
            DS_DELETE,
            array(
                'access_token' => $this->opends_token,
                'ds_id' => $ds_id
            )
        );
        return TRUE;
    }

    public function create_tb($ds_id, $name, $schema, $uniq_key, $title) {
        $res = $this->curl_tool->Post(
            TB_CREATE,
            array('access_token' => $this->opends_token),
            array(
                'ds_id' => $ds_id,
                'name' => $name,
                'schema' => $schema,
                'uniq_key' => $uniq_key,
                'title' => $title
            )
        );
        return $res['tb_id'];
    }

    public function clean_tb($tb_id) {
        $res = $this->curl_tool->Post(
            TB_CLEAN,
            array('access_token' => $this->opends_token, 'tb_id' => $tb_id),
            array()
        );
        return TRUE;
    }

    public function insert_data($tb_id, $fields, $data, $by_fid) {

        if ($by_fid) {
            $send_f = array(
                'access_token' => $this->opends_token,
                'fields_id' => json_encode($fields),
                'tb_id' => $tb_id
            );
        }
        else {
            $send_f = array(
                'access_token' => $this->opends_token,
                'fields' => json_encode($fields),
                'tb_id' => $tb_id
            );
        }

        $res = $this->curl_tool->Post(
            TB_DATA_INSERT,
            $send_f,
            $data
        );
        return TRUE;
    }

    public function delete_tb($tb_id) {
        $res = $this->curl_tool->Post(
            TB_DELETE,
            array('access_token' => $this->opends_token, 'tb_id' => $tb_id),
            array()
        );
        return TRUE;
    }

    public function preview_tb($tb_id) {
        $res = $this->curl_tool->Post(
            TB_PREVIEW,
            array('access_token' => $this->opends_token, 'tb_id' => $tb_id),
            array()
        );
        return $res;
    }
    public function commit_tb($tb_id) {
        $res = $this->curl_tool->Post(
            TB_COMMIT,
            array('access_token' => $this->opends_token, 'tb_id' => $tb_id),
            array()
        );
        return TRUE;
    }

    public function revert_tb($tb_id) {
        $res = $this->curl_tool->Post(
            TB_REVERT,
            array('access_token' => $this->opends_token, 'tb_id' => $tb_id),
            array()
        );
        return TRUE;
    }

    public function update_tb($tb_ids) {
        $res = $this->curl_tool->Post(
            TB_UPDATE,
            array('access_token' => $this->opends_token, 'tb_ids' => json_encode($tb_ids)),
            array()
        );
        return TRUE;
    }
    public function bulk_delete_tb($tb_id, $where) {
        $res = $this->curl_tool->Post(
            TB_BULKDELETE,
            array(
                'access_token' => $this->opends_token,
                'tb_id' => $tb_id,
                'where' => $where
            ),
            array()
        );
        return TRUE;
    }

    public function delete_data($tb_id, $fields, $data, $by_fid) {
        if ($by_fid) {
            $send_f = array(
                'access_token' => $this->opends_token,
                'fields_id' => json_encode($fields),
                'tb_id' => $tb_id
            );
        }
        else {
            $send_f = array(
                'access_token' => $this->opends_token,
                'fields' => json_encode($fields),
                'tb_id' => $tb_id
            );
        }

        $res = $this->curl_tool->Post(
            TB_DATA_DELETE,
            $send_f,
            $data
        );
        return TRUE;
    }

    public function update_data($tb_id, $fields, $data, $by_fid) {

        if ($by_fid) {
            $send_f = array(
                'access_token' => $this->opends_token,
                'fields_id' => json_encode($fields),
                'tb_id' => $tb_id
            );
        }
        else {
            $send_f = array(
                'access_token' => $this->opends_token,
                'fields' => json_encode($fields),
                'tb_id' => $tb_id
            );
        }

        $res = $this->curl_tool->Post(
            TB_DATA_UPDATE,
            $send_f,
            $data
        );
        return TRUE;
    }

    public function modify_tb($tb_id, $alias_name) {
        $res = $this->curl_tool->Post(
            TB_MODIFY,
            array(
                'access_token' => $this->opends_token,
                'tb_id' => $tb_id,
                'name' => $alias_name
            ),
            array()
        );
        return TRUE;
    }
    public function list_tb($ds_id) {
        $res = $this->curl_tool->Post(
            TB_LIST,
            array(
                'access_token' => $this->opends_token,
                'ds_id' => $ds_id
            ),
            array()
        );
        return TRUE;
    }

    public function add_field($tb_id, $field_name, $field_type, $uniq_index, $title=NULL) {

        $params = array(
            'access_token' => $this->opends_token,
            'tb_id' => $tb_id,
            'name' => $field_name,
            'type' => $field_type,
            'uniq_index' => $uniq_index
        );
        if ($title != NULL) {
            $params["title"] = $title;
        }
        $res = $this->curl_tool->Post(
            FD_ADD,
            $params,
            array()
        );
        return TRUE;
    }

    public function delete_field($tb_id, $field_name) {
        $res = $this->curl_tool->Post(
            FD_DELETE,
            array(
                'access_token' => $this->opends_token,
                'tb_id' => $tb_id,
                'name' => $field_name
            ),
            array()
        );
        return TRUE;
    }

    public function modify_field($tb_id, $field_name, $field_type, $uniq_index, $title) {
        $params = array(
            'access_token' => $this->opends_token,
            'tb_id' => $tb_id,
            'name' => $field_name,
            'type' => $field_type,
            'uniq_index' => $uniq_index
        );
        if ($title != NULL) {
            $params["title"] = $title;
        }

        $res = $this->curl_tool->Post(
            FD_MODIFY,
            $params,
            array()
        );
        return TRUE;
    }
    public function List_field($tb_id) {
        $res = $this->curl_tool->Post(
            FD_LIST,
            array('access_token' => $this->opends_token, 'tb_id' => $tb_id),
            array()
        );
        return $res;
    }

    public function get_info($tb_id) {
        $res = $this->curl_tool->Post(
            TB_INFO,
            array('access_token' => $this->opends_token, 'tb_id' => $tb_id),
            array()
        );
        return $res;
    }

    public function service_uri($domain) {
        $res = $this->curl_tool->Get(
            SERVICE_URI,
            array('access_token' => $this->opends_token, 'domain' => $domain)
        );
        return $res;
    }
}

class Table {
    private $name = NULL;
    private $ds = NULL;
    public  $tb_id = NULL;
    private $is_new_table = NULL;
    private $schema = NULL;
    private $uniq_key = NULL;

    public function __construct($ds, $name, $tb_id, $schema, $uniq_key, $title) {
        $this->name = $name;
        $this->ds = $ds;
        $this->tb_id = $tb_id;
        $this->schema = $schema;
        $this->uniq_key = $uniq_key;

        if ($tb_id == NULL) {
            $this->tb_id = $this->ds->opends->create_tb($this->ds->ds_id, $name, $schema, $uniq_key, $title);

            $this->is_new_table = 1;
        }
        else {

            $this->is_new_table = 0;
        }

    }
    public function get_id() {
        return $this->tb_id;
    }
    public function get_name() {
        return $this->name;
    }

    public function insert_data_by_name($fields, $data) {
        $this->ds->opends->insert_data($this->tb_id, $fields, $data, 0);

        return TRUE;
    }

    public function insert_data_by_id($fields, $data) {
        $this->ds->opends->insert_data($this->tb_id, $fields, $data, 1);

        return TRUE;
    }


    public function bulk_delete($where) {
        $this->ds->opends->bulk_delete_tb($this->tb_id, $where);
        return TRUE;
    }
    public function modify_table($alias_name) {
        if($alias_name == NULL or $alias_name == "") {
            throw new Exception("table alias name can't be NULL or ''");
        }
        $this->ds->opends->modify_tb($this->tb_id, $alias_name);
        return TRUE;
    }
    public function get_fields() {
        return $this->ds->opends->list_field($this->tb_id);
    }
    public function add_field($field_name, $field_type, $uniq_index=0, $title=NULL) {
        if (in_array($field_type, array("string", "number", "date")) == FALSE) {
            throw new Exception("wrong field_type");
        }
        if ($uniq_index != 0 and $uniq_index != 1) {
            throw new Exception("uniq_index can only be 0 or 1");
        }
        $this->ds->opends->add_field($this->tb_id, $field_name, $field_type, $uniq_index, $title);
        return TRUE;
    }
    public function delete_field($field_name) {
        $this->ds->opends->delete_field($this->tb_id, $field_name);
        return TRUE;
    }
    public function modify_field($field_name, $field_type, $uniq_index=0, $title=NULL) {
        if (in_array($field_type, array("string", "number", "date")) == FALSE) {
            throw new Exception("wrong field_type");
        }
        if ($uniq_index != 0 and $uniq_index != 1) {
            throw new Exception("uniq_index can only be 0 or 1");
        }
        $this->ds->opends->modify_field($this->tb_id, $field_name, $field_type, $uniq_index, $title);
        return TRUE;
    }

    public function clean() {
        $this->ds->opends->clean_tb($this->tb_id);
        return TRUE;
    }

    public function preview() {
        return $this->ds->opends->preview_tb($this->tb_id);
    }

    public function commit() {
        $this->ds->opends->commit_tb($this->tb_id);
        return TRUE;
    }

    public function revert() {
        $this->ds->opends->revert_tb($this->tb_id);
        return TRUE;
    }

    public function delete_data_by_name($fields, $data) {
        $this->ds->opends->delete_data($this->tb_id, $fields, $data, 0);
        return TRUE;
    }

    public function delete_data_by_id($fields, $data) {
        $this->ds->opends->delete_data($this->tb_id, $fields, $data, 1);
        return TRUE;
    }

    public function update_data_by_name($fields, $data) {
        $this->ds->opends->update_data($this->tb_id, $fields, $data, 0);
        return TRUE;
    }

    public function update_data_by_id($fields, $data) {
        $this->ds->opends->update_data($this->tb_id, $fields, $data, 1);
        return TRUE;
    }

    public function get_info() {
        return $this->ds->opends->get_info($this->tb_id);
    }

}

class DS {
    public $opends = NULL;
    public $name = NULL;
    public $ds_id = NULL;
    private $tables = array();
    public $tb_ids = array();

    public function __construct($name, $client) {
        $this->opends = $client->opends;
        $this->name = $name;
        $ds_list = $this->opends->list_ds();



        foreach ($ds_list as $ds) {
            if ($this->name == $ds["name"]) {
                $this->ds_id = $ds["ds_id"];
                foreach ($ds["tables"] as $table) {
                    $this->tables[$table[0]] = new Table($this, $table[0], $table[1], NULL, NULL, NULL);
                }
                break;
            }
        }



        if ($this->ds_id == NULL) {
            $this->ds_id = $this->opends->create_ds($name);
        }

    }

    public function get_all_tables() {
        return $this->tables;
    }
    public function get_table($name) {
        $res = $this->tables[$name];

        if ($res == NULL) {
            throw new Exception("table not exists");
        }
        return $res;
    }
    public function get_name() {
        return $this->name;
    }
    public function get_id(){
        return $this->ds_id;
    }

    public function create_table($name, $schema, $uniq_key=NULL, $title=NULL) {
        if (is_array($schema) == FALSE) {
            throw new Exception("schema is not array");
        }

        if ($this->tables[$name] != NUll) {
            throw new Exception("table already exists");
        }
        else {
            if ($schema != NULL && $schema[0]["name"] == NULL) {
                $schema = $this->gen_schema($schema);
            }

            $this->tables[$name] = new Table($this, $name, NULL, $schema, $uniq_key, $title);
            return $this->tables[$name];
        }
    }

    private function gen_schema($old_schema) {
        $tmp = array();
        foreach ($old_schema as $old) {
            if (count($old) == 2) {
                $tmp[] = array("name" => $old[0], "type" => $old[1]);
            }
            if (count($old) == 3) {
                $tmp[] = array("name" => $old[0], "type" => $old[1], "comment" => $old[2]);
            }
            if (count($old) == 4) {
                $tmp[] = array("name" => $old[0], "type" => $old[1], "comment" => $old[2], "title" => $old[3]);
            }
        }
        return $tmp;
    }

    public function delete_table($name) {
        if (is_string($name) == FALSE) {
            throw new Exception("table name is not string");
        }
        if ($this->get_table($name) != NULL) {
            $tb_id = $this->get_table($name)->tb_id;
            $this->opends->delete_tb($tb_id);
            unset($this->tables[$name]);
        }
        else {
            throw new Exception("table not exists");
        }
    }

    public function delete() {
        $this->opends->delete_ds($this->ds_id);
    }

    public function get_tb_ids () {
        $tb_ids = array();
        foreach ($this->tables as $key => $value) {
            $tb_ids[] = $value->tb_id;
        }
        return $tb_ids;
    }

    public function update($tb_ids) {
        if ($tb_ids == NULL) {
            throw new Exception("tb_ids can't be NULL");
        }
        if (count($tb_ids) == 0) {
            echo "no table updated!";
            return FALSE;
        }
        $this->opends->update_tb($tb_ids);
    }

    public function update_all() {
        $tb_ids = $this->get_tb_ids();
        $this->opends->update_tb($tb_ids);
    }

}

class BDPClient {
    private $token = NULL;
    public $opends = NULL;
    public $dss = array();
    public function __construct($token, $domain) {
        $this->token = $token;
        $this->opends = new OPENDS($token, $domain);
        $ds_list = $this->opends->list_ds();
        foreach ($ds_list as $value) {
            try {
                $this->dss[$value["name"]] = new DS($value["name"], $this);
            } catch (Exception $e) {
                throw new Exception("initial ds failed");
            }
        }
    }

    public function get_all_ds() {
        return $this->dss;
    }

    public function create_ds($name) {
        if ($this->dss[$name] == NULL) {
            $this->dss[$name] = new DS($name, $this);
        }
        else {
            throw new Exception("ds already exists");
        }
        return $this->dss[$name];
    }

    public function delete_ds($name) {
        if ($this->dss[$name] != NULL) {
            $this->opends->delete_ds($this->dss[$name]->ds_id);
            unset($this->dss[$name]);
        }
        else {
            throw new Exception("ds not exists");
        }
    }

    public function get_ds($name) {
        if($name == '' or $name == NULL) {
            throw new Exception("ds name can't be '' or NULL");
        }
        if ($this->dss[$name] == NULL) {
            throw new Exception("ds not exists");
        }
        return $this->dss[$name];
    }
}
